@@ -175,8 +175,7 @@ _gst_caps_copy (const GstCaps * caps)
   GST_CAPS_FLAGS (newcaps) = GST_CAPS_FLAGS (caps);
   n = GST_CAPS_LEN (caps);
 
-  GST_CAT_DEBUG_OBJECT (GST_CAT_PERFORMANCE, caps, "doing copy %p -> %p",
-      caps, newcaps);
+  GST_CAT_DEBUG (GST_CAT_PERFORMANCE, "doing copy %p -> %p", caps, newcaps);
 
   for (i = 0; i < n; i++) {
     structure = gst_caps_get_structure_unchecked (caps, i);
@@ -511,6 +510,20 @@ gst_caps_remove_and_get_structure (GstCaps * caps, guint idx)
   return s;
 }
 
+static void
+gst_caps_make_any (GstCaps * caps)
+{
+  guint i;
+  GstStructure *s;
+
+  /* empty out residual structures */
+  for (i = GST_CAPS_LEN (caps); i; i--) {
+    s = gst_caps_remove_and_get_structure (caps, 0);
+    gst_structure_free (s);
+  }
+  GST_CAPS_FLAGS (caps) |= GST_CAPS_FLAG_ANY;
+}
+
 /**
  * gst_caps_steal_structure:
  * @caps: the #GstCaps to retrieve from
@@ -555,7 +568,7 @@ gst_caps_append (GstCaps * caps1, GstCaps * caps2)
   g_return_if_fail (IS_WRITABLE (caps1));
 
   if (G_UNLIKELY (CAPS_IS_ANY (caps1) || CAPS_IS_ANY (caps2))) {
-    GST_CAPS_FLAGS (caps1) |= GST_CAPS_FLAG_ANY;
+    gst_caps_make_any (caps1);
     gst_caps_unref (caps2);
   } else {
     caps2 = gst_caps_make_writable (caps2);
@@ -637,6 +650,13 @@ gst_caps_append_structure (GstCaps * caps, GstStructure * structure)
   g_return_if_fail (GST_IS_CAPS (caps));
   g_return_if_fail (IS_WRITABLE (caps));
 
+  if (CAPS_IS_ANY (caps)) {
+    /* ANY caps will stay as ANY caps */
+    if (structure)
+      gst_structure_free (structure);
+    return;
+  }
+
   if (G_LIKELY (structure)) {
     gst_caps_append_structure_unchecked (caps, structure, NULL);
   }
@@ -660,6 +680,15 @@ gst_caps_append_structure_full (GstCaps * caps, GstStructure * structure,
   g_return_if_fail (GST_IS_CAPS (caps));
   g_return_if_fail (IS_WRITABLE (caps));
 
+  if (CAPS_IS_ANY (caps)) {
+    /* ANY caps will stay as ANY caps */
+    if (structure)
+      gst_structure_free (structure);
+    if (features)
+      gst_caps_features_free (features);
+    return;
+  }
+
   if (G_LIKELY (structure)) {
     gst_caps_append_structure_unchecked (caps, structure, features);
   }
@@ -679,7 +708,7 @@ gst_caps_remove_structure (GstCaps * caps, guint idx)
   GstStructure *structure;
 
   g_return_if_fail (caps != NULL);
-  g_return_if_fail (idx <= gst_caps_get_size (caps));
+  g_return_if_fail (idx < gst_caps_get_size (caps));
   g_return_if_fail (IS_WRITABLE (caps));
 
   structure = gst_caps_remove_and_get_structure (caps, idx);
@@ -708,6 +737,12 @@ gst_caps_merge_structure (GstCaps * caps, GstStructure * structure)
   if (G_UNLIKELY (structure == NULL))
     return caps;
 
+  if (CAPS_IS_ANY (caps)) {
+    /* ANY caps will stay as ANY caps */
+    gst_structure_free (structure);
+    return caps;
+  }
+
   /* check each structure */
   for (i = GST_CAPS_LEN (caps) - 1; i >= 0; i--) {
     structure1 = gst_caps_get_structure_unchecked (caps, i);
@@ -759,6 +794,14 @@ gst_caps_merge_structure_full (GstCaps * caps, GstStructure * structure,
   if (G_UNLIKELY (structure == NULL))
     return caps;
 
+  if (CAPS_IS_ANY (caps)) {
+    /* ANY caps will stay as ANY caps */
+    gst_structure_free (structure);
+    if (features)
+      gst_caps_features_free (features);
+    return caps;
+  }
+
   /* To make comparisons easier below */
   features_tmp = features ? features : GST_CAPS_FEATURES_MEMORY_SYSTEM_MEMORY;
 
@@ -886,7 +929,8 @@ gst_caps_get_features (const GstCaps * caps, guint index)
     gst_caps_features_set_parent_refcount (features, &GST_CAPS_REFCOUNT (caps));
 
     storage = gst_caps_get_features_storage_unchecked (caps, index);
-    if (!g_atomic_pointer_compare_and_exchange (storage, NULL, features)) {
+    if (!g_atomic_pointer_compare_and_exchange (storage,
+            (GstCapsFeatures *) NULL, features)) {
       /* Someone did the same we just tried in the meantime */
       gst_caps_features_set_parent_refcount (features, NULL);
       gst_caps_features_free (features);
@@ -915,7 +959,7 @@ gst_caps_set_features (GstCaps * caps, guint index, GstCapsFeatures * features)
   GstCapsFeatures **storage, *old;
 
   g_return_if_fail (caps != NULL);
-  g_return_if_fail (index <= gst_caps_get_size (caps));
+  g_return_if_fail (index < gst_caps_get_size (caps));
   g_return_if_fail (IS_WRITABLE (caps));
 
   storage = gst_caps_get_features_storage_unchecked (caps, index);
@@ -952,6 +996,13 @@ gst_caps_set_features_simple (GstCaps * caps, GstCapsFeatures * features)
 
   n = gst_caps_get_size (caps);
 
+  if (n == 0) {
+    /* features will not be set on any structure */
+    if (features)
+      gst_caps_features_free (features);
+    return;
+  }
+
   for (i = 0; i < n; i++) {
     GstCapsFeatures *f;
 
@@ -974,6 +1025,8 @@ gst_caps_set_features_simple (GstCaps * caps, GstCapsFeatures * features)
  * contained in @caps.
  *
  * Returns: (transfer full): the new #GstCaps
+ *
+ * Since: 1.16
  */
 GstCaps *
 gst_caps_copy_nth (const GstCaps * caps, guint nth)
@@ -1009,6 +1062,10 @@ gst_caps_copy_nth (const GstCaps * caps, guint nth)
  * on it if necessary, so you must not use @caps afterwards unless you keep an
  * additional reference to it with gst_caps_ref().
  *
+ * Note that it is not guaranteed that the returned caps have exactly one
+ * structure. If @caps is any or empty caps then then returned caps will be
+ * the same and contain no structure at all.
+ *
  * Returns: (transfer full): truncated caps
  */
 GstCaps *
@@ -1018,7 +1075,13 @@ gst_caps_truncate (GstCaps * caps)
 
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
 
+  /* Nothing to truncate here */
+  if (GST_CAPS_LEN (caps) == 0)
+    return caps;
+
   i = GST_CAPS_LEN (caps) - 1;
+
+  /* Already truncated */
   if (i == 0)
     return caps;
 
@@ -1181,6 +1244,10 @@ gst_caps_is_fixed (const GstCaps * caps)
   if (GST_CAPS_LEN (caps) != 1)
     return FALSE;
 
+  /* double check not ANY, even though ANY caps should have 0 length */
+  if (CAPS_IS_ANY (caps))
+    return FALSE;
+
   features = gst_caps_get_features_unchecked (caps, 0);
   if (features && gst_caps_features_is_any (features))
     return FALSE;
@@ -1268,11 +1335,12 @@ gst_caps_is_subset (const GstCaps * subset, const GstCaps * superset)
     return FALSE;
 
   for (i = GST_CAPS_LEN (subset) - 1; i >= 0; i--) {
+    s1 = gst_caps_get_structure_unchecked (subset, i);
+    f1 = gst_caps_get_features_unchecked (subset, i);
+    if (!f1)
+      f1 = GST_CAPS_FEATURES_MEMORY_SYSTEM_MEMORY;
+
     for (j = GST_CAPS_LEN (superset) - 1; j >= 0; j--) {
-      s1 = gst_caps_get_structure_unchecked (subset, i);
-      f1 = gst_caps_get_features_unchecked (subset, i);
-      if (!f1)
-        f1 = GST_CAPS_FEATURES_MEMORY_SYSTEM_MEMORY;
       s2 = gst_caps_get_structure_unchecked (superset, j);
       f2 = gst_caps_get_features_unchecked (superset, j);
       if (!f2)
@@ -1285,6 +1353,7 @@ gst_caps_is_subset (const GstCaps * subset, const GstCaps * superset)
         break;
       }
     }
+
     /* If we found no superset for this subset structure
      * we return FALSE immediately */
     if (j == -1) {
@@ -1426,6 +1495,12 @@ gst_caps_is_strictly_equal (const GstCaps * caps1, const GstCaps * caps2)
   if (G_UNLIKELY (caps1 == caps2))
     return TRUE;
 
+  /* if both are ANY caps, consider them strictly equal */
+  if (CAPS_IS_ANY (caps1))
+    return (CAPS_IS_ANY (caps2));
+  else if (CAPS_IS_ANY (caps2))
+    return FALSE;
+
   if (GST_CAPS_LEN (caps1) != GST_CAPS_LEN (caps2))
     return FALSE;
 
@@ -1548,21 +1623,6 @@ gst_caps_intersect_zig_zag (GstCaps * caps1, GstCaps * caps2)
   GstCaps *dest;
   GstStructure *istruct;
 
-  /* caps are exactly the same pointers, just copy one caps */
-  if (G_UNLIKELY (caps1 == caps2))
-    return gst_caps_ref (caps1);
-
-  /* empty caps on either side, return empty */
-  if (G_UNLIKELY (CAPS_IS_EMPTY (caps1) || CAPS_IS_EMPTY (caps2)))
-    return gst_caps_ref (GST_CAPS_NONE);
-
-  /* one of the caps is any, just copy the other caps */
-  if (G_UNLIKELY (CAPS_IS_ANY (caps1)))
-    return gst_caps_ref (caps2);
-
-  if (G_UNLIKELY (CAPS_IS_ANY (caps2)))
-    return gst_caps_ref (caps1);
-
   dest = gst_caps_new_empty ();
   /* run zigzag on top line then right line, this preserves the caps order
    * much better than a simple loop.
@@ -1647,21 +1707,6 @@ gst_caps_intersect_first (GstCaps * caps1, GstCaps * caps2)
   GstCaps *dest;
   GstStructure *istruct;
 
-  /* caps are exactly the same pointers, just copy one caps */
-  if (G_UNLIKELY (caps1 == caps2))
-    return gst_caps_ref (caps1);
-
-  /* empty caps on either side, return empty */
-  if (G_UNLIKELY (CAPS_IS_EMPTY (caps1) || CAPS_IS_EMPTY (caps2)))
-    return gst_caps_ref (GST_CAPS_NONE);
-
-  /* one of the caps is any, just copy the other caps */
-  if (G_UNLIKELY (CAPS_IS_ANY (caps1)))
-    return gst_caps_ref (caps2);
-
-  if (G_UNLIKELY (CAPS_IS_ANY (caps2)))
-    return gst_caps_ref (caps1);
-
   dest = gst_caps_new_empty ();
   len1 = GST_CAPS_LEN (caps1);
   len2 = GST_CAPS_LEN (caps2);
@@ -1713,6 +1758,22 @@ gst_caps_intersect_full (GstCaps * caps1, GstCaps * caps2,
   g_return_val_if_fail (GST_IS_CAPS (caps1), NULL);
   g_return_val_if_fail (GST_IS_CAPS (caps2), NULL);
 
+  /* Common fast-path */
+  /* caps are exactly the same pointers, just copy one caps */
+  if (G_UNLIKELY (caps1 == caps2))
+    return gst_caps_ref (caps1);
+
+  /* empty caps on either side, return empty */
+  if (G_UNLIKELY (CAPS_IS_EMPTY (caps1) || CAPS_IS_EMPTY (caps2)))
+    return gst_caps_ref (GST_CAPS_NONE);
+
+  /* one of the caps is any, just copy the other caps */
+  if (G_UNLIKELY (CAPS_IS_ANY (caps1)))
+    return gst_caps_ref (caps2);
+
+  if (G_UNLIKELY (CAPS_IS_ANY (caps2)))
+    return gst_caps_ref (caps1);
+
   switch (mode) {
     case GST_CAPS_INTERSECT_FIRST:
       return gst_caps_intersect_first (caps1, caps2);
@@ -2121,6 +2182,10 @@ gst_caps_simplify (GstCaps * caps)
 
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
 
+  /* empty/any caps are as simple as can be */
+  if (GST_CAPS_LEN (caps) == 0)
+    return caps;
+
   start = GST_CAPS_LEN (caps) - 1;
   /* one caps, already as simple as can be */
   if (start == 0)
@@ -2182,6 +2247,12 @@ gst_caps_simplify (GstCaps * caps)
  * on it so you must not use @caps afterwards unless you keep an additional
  * reference to it with gst_caps_ref().
  *
+ * Note that it is not guaranteed that the returned caps have exactly one
+ * structure. If @caps are empty caps then then returned caps will be
+ * the empty too and contain no structure at all.
+ *
+ * Calling this function with any caps is not allowed.
+ *
  * Returns: (transfer full): the fixated caps
  */
 GstCaps *
@@ -2191,10 +2262,20 @@ gst_caps_fixate (GstCaps * caps)
   GstCapsFeatures *f;
 
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
+  g_return_val_if_fail (!CAPS_IS_ANY (caps), NULL);
 
   /* default fixation */
   caps = gst_caps_truncate (caps);
   caps = gst_caps_make_writable (caps);
+
+  /* need to return early here because empty caps have no structure
+   * but must return after make_writable() because the documentation
+   * specifies that it will call make_writable() on the return value
+   * and callers might assume writable caps.
+   */
+  if (CAPS_IS_EMPTY (caps))
+    return caps;
+
   s = gst_caps_get_structure (caps, 0);
   gst_structure_fixate (s);
 
