@@ -21,6 +21,7 @@
 
 /**
  * SECTION:gsttaglist
+ * @title: GstTagList
  * @short_description: List of tags and values used to describe media metadata
  *
  * List of tags and values used to describe media metadata.
@@ -470,13 +471,13 @@ gst_tag_lookup (const gchar * tag_name)
 }
 
 /**
- * gst_tag_register:
+ * gst_tag_register: (skip)
  * @name: the name or identifier string
  * @flag: a flag describing the type of tag info
  * @type: the type this data is in
  * @nick: human-readable name
  * @blurb: a human-readable description about this tag
- * @func: (allow-none) (scope call): function for merging multiple values of this tag, or %NULL
+ * @func: (allow-none): function for merging multiple values of this tag, or %NULL
  *
  * Registers a new tag type for the use with GStreamer's type system. If a type
  * with that name is already registered, that one is used.
@@ -509,18 +510,18 @@ gst_tag_register (const gchar * name, GstTagFlag flag, GType type,
   g_return_if_fail (blurb != NULL);
   g_return_if_fail (type != 0 && type != GST_TYPE_LIST);
 
-  return gst_tag_register_static (g_intern_string (name), flag, type,
+  gst_tag_register_static (g_intern_string (name), flag, type,
       g_intern_string (nick), g_intern_string (blurb), func);
 }
 
 /**
- * gst_tag_register_static:
+ * gst_tag_register_static: (skip)
  * @name: the name or identifier string (string constant)
  * @flag: a flag describing the type of tag info
  * @type: the type this data is in
  * @nick: human-readable name or short description (string constant)
  * @blurb: a human-readable description for this tag (string constant)
- * @func: (allow-none) (scope call): function for merging multiple values of this tag, or %NULL
+ * @func: (allow-none): function for merging multiple values of this tag, or %NULL
  *
  * Registers a new tag type for the use with GStreamer's type system.
  *
@@ -603,7 +604,7 @@ gst_tag_get_type (const gchar * tag)
  * Returns the human-readable name of this tag, You must not change or free
  * this string.
  *
- * Returns: the human-readable name of this tag
+ * Returns: (nullable): the human-readable name of this tag
  */
 const gchar *
 gst_tag_get_nick (const gchar * tag)
@@ -612,7 +613,11 @@ gst_tag_get_nick (const gchar * tag)
 
   g_return_val_if_fail (tag != NULL, NULL);
   info = gst_tag_lookup (tag);
-  g_return_val_if_fail (info != NULL, NULL);
+  if (!info) {
+    GST_WARNING ("Unknown tag: %s", tag);
+
+    return tag;
+  }
 
   return info->nick;
 }
@@ -624,7 +629,7 @@ gst_tag_get_nick (const gchar * tag)
  * Returns the human-readable description of this tag, You must not change or
  * free this string.
  *
- * Returns: the human-readable description of this tag
+ * Returns: (nullable): the human-readable description of this tag
  */
 const gchar *
 gst_tag_get_description (const gchar * tag)
@@ -714,6 +719,10 @@ __gst_tag_list_free (GstTagList * list)
 
   gst_structure_free (GST_TAG_LIST_STRUCTURE (list));
 
+#ifdef USE_POISONING
+  memset (list, 0xff, sizeof (GstTagListImpl));
+#endif
+
   g_slice_free1 (sizeof (GstTagListImpl), list);
 }
 
@@ -945,24 +954,35 @@ gst_tag_list_is_empty (const GstTagList * list)
 }
 
 static gboolean
-gst_tag_list_fields_equal (const GValue * value1, const GValue * value2)
+gst_tag_list_fields_equal (GQuark field_id, const GValue * value2,
+    gpointer data)
 {
+  const GstStructure *struct1 = (const GstStructure *) data;
+  const GValue *value1 = gst_structure_id_get_value (struct1, field_id);
   gdouble d1, d2;
 
+  if (value1 == NULL) {
+    /* no value with this field id, clearly not equal */
+    return FALSE;
+  }
+
   if (gst_value_compare (value1, value2) == GST_VALUE_EQUAL)
     return TRUE;
 
   /* fields not equal: add some tolerance for doubles, otherwise bail out */
-  if (!G_VALUE_HOLDS_DOUBLE (value1) || !G_VALUE_HOLDS_DOUBLE (value2))
-    return FALSE;
+  if ((G_VALUE_TYPE (value1) == G_VALUE_TYPE (value2)) &&
+      G_VALUE_TYPE (value1) == G_TYPE_DOUBLE) {
+    d1 = g_value_get_double (value1);
+    d2 = g_value_get_double (value2);
 
-  d1 = g_value_get_double (value1);
-  d2 = g_value_get_double (value2);
+    /* This will only work for 'normal' values and values around 0,
+     * which should be good enough for our purposes here
+     * FIXME: maybe add this to gst_value_compare_double() ? */
+    return (fabs (d1 - d2) < 0.0000001);
 
-  /* This will only work for 'normal' values and values around 0,
-   * which should be good enough for our purposes here
-   * FIXME: maybe add this to gst_value_compare_double() ? */
-  return (fabs (d1 - d2) < 0.0000001);
+  }
+
+  return FALSE;
 }
 
 /**
@@ -978,7 +998,6 @@ gboolean
 gst_tag_list_is_equal (const GstTagList * list1, const GstTagList * list2)
 {
   const GstStructure *s1, *s2;
-  gint num_fields1, num_fields2, i;
 
   g_return_val_if_fail (GST_IS_TAG_LIST (list1), FALSE);
   g_return_val_if_fail (GST_IS_TAG_LIST (list2), FALSE);
@@ -989,28 +1008,14 @@ gst_tag_list_is_equal (const GstTagList * list1, const GstTagList * list2)
   s1 = GST_TAG_LIST_STRUCTURE (list1);
   s2 = GST_TAG_LIST_STRUCTURE (list2);
 
-  num_fields1 = gst_structure_n_fields (s1);
-  num_fields2 = gst_structure_n_fields (s2);
+  if (G_UNLIKELY (s1 == s2))
+    return TRUE;
 
-  if (num_fields1 != num_fields2)
+  if (gst_structure_n_fields (s1) != gst_structure_n_fields (s2)) {
     return FALSE;
-
-  for (i = 0; i < num_fields1; i++) {
-    const GValue *value1, *value2;
-    const gchar *tag_name;
-
-    tag_name = gst_structure_nth_field_name (s1, i);
-    value1 = gst_structure_get_value (s1, tag_name);
-    value2 = gst_structure_get_value (s2, tag_name);
-
-    if (value2 == NULL)
-      return FALSE;
-
-    if (!gst_tag_list_fields_equal (value1, value2))
-      return FALSE;
   }
 
-  return TRUE;
+  return gst_structure_foreach (s1, gst_tag_list_fields_equal, (gpointer) s2);
 }
 
 typedef struct
@@ -1294,7 +1299,7 @@ gst_tag_list_add_valist (GstTagList * list, GstTagMergeMode mode,
       g_warning ("%s: %s", G_STRLOC, error);
       g_free (error);
       /* we purposely leak the value here, it might not be
-       * in a sane state if an error condition occoured
+       * in a sane state if an error condition occurred
        */
       return;
     }
@@ -1634,6 +1639,18 @@ TAG_MERGE_FUNCS (int, gint, TRUE);
  *              given list.
  */
 TAG_MERGE_FUNCS (uint, guint, TRUE);
+/**
+ * gst_tag_list_get_int64:
+ * @list: a #GstTagList to get the tag from
+ * @tag: tag to read out
+ * @value: (out): location for the result
+ *
+ * Copies the contents for the given tag into the value, merging multiple values
+ * into one if multiple values are associated with the tag.
+ *
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
+ *              given list.
+ */
 /**
  * gst_tag_list_get_int64_index:
  * @list: a #GstTagList to get the tag from
@@ -2045,3 +2062,24 @@ gst_tag_list_get_sample_index (const GstTagList * list,
   *sample = g_value_dup_boxed (v);
   return (*sample != NULL);
 }
+
+/**
+ * gst_tag_list_copy:
+ * @taglist: a #GstTagList.
+ *
+ * Creates a new #GstTagList as a copy of the old @taglist. The new taglist
+ * will have a refcount of 1, owned by the caller, and will be writable as
+ * a result.
+ *
+ * Note that this function is the semantic equivalent of a gst_tag_list_ref()
+ * followed by a gst_tag_list_make_writable(). If you only want to hold on to a
+ * reference to the data, you should use gst_tag_list_ref().
+ *
+ * When you are finished with the taglist, call gst_tag_list_unref() on it.
+ *
+ * Returns: the new #GstTagList
+ */
+GstTagList *(gst_tag_list_copy) (const GstTagList * taglist)
+{
+  return GST_TAG_LIST (gst_mini_object_copy (GST_MINI_OBJECT_CAST (taglist)));
+}
